/*
 *  TU Eindhoven
 *  Eindhoven, The Netherlands
 *
 *  Name            :   sequence.h
 *
 *  Author          :   Sander Stuijk (sander@ics.ele.tue.nl)
 *
 *  Date            :   May 7, 2007
 *
 *  Function        :   Sequence class
 *
 *  History         :
 *      07-05-07    :   Initial version.
 *
 * $Id: sequence.h,v 1.1.1.1 2007/10/02 10:59:48 sander Exp $
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 * In other words, you are welcome to use, share and improve this program.
 * You are forbidden to forbid anyone else to use, share and improve
 * what you give them.   Happy coding!
 */

#ifndef BASE_SEQUENCE_SEQUENCE_H_INCLUDED
#define BASE_SEQUENCE_SEQUENCE_H_INCLUDED

#include "../basic_types.h"

/**
 * CSequence
 * Sequence class template
 */
template <class T>
class CSequence
{
public:
    // Constructor
    CSequence() { pos = 0; };
    CSequence(T t) { values.push_back(t); pos = 0; };
    CSequence(vector<T> &v) { values = v; pos = 0; };
    CSequence(CStrings &v) { 
        pos = 0;
        for (CStringsIter iter = v.begin(); iter != v.end(); iter++)
            values.push_back(*iter);
    };
        
    // Destructor
    ~CSequence() {};
    
    // Position within sequence
    uint next() { pos = (pos + 1) % size(); return pos; };
    uint previous() { pos = (pos == 0 ? size() - 1 : pos - 1); return pos; };
    void reset() { pos = 0; };
    uint position() const { return pos; };
    
    // Value at position
    T value() const { return values[pos]; };
    T value(uint i) const { return values[i % size()]; };
    T &back() { return values[size() - 1]; };
    
    // Size of the sequence
    uint size() const { return values.size(); };

    // Modification of the sequence
    void push_back(T t) { values.push_back(t); pos = 0; };
    
    // Print sequence
    ostream &print(ostream &out)
    {
        for (uint i = 0; i < size(); i++)
        {
            if (i != 0)
                out << ",";
            out << values[i];
        }
                
        return out;
    }

    friend ostream &operator<<(ostream &out, CSequence &s)
        { return s.print(out); };
    
private:
    uint pos;
    vector<T> values;
};

#endif
