/*
 *  TU Eindhoven
 *  Eindhoven, The Netherlands
 *
 *  Name            :   cluster.h
 *
 *  Author          :   Sander Stuijk (sander@ics.ele.tue.nl)
 *
 *  Date            :   July 19, 2005
 *
 *  Function        :   SDF actor cluster
 *
 *  History         :
 *      19-07-05    :   Initial version.
 *
 * $Id: cluster.h,v 1.1.1.1 2007/10/02 10:59:46 sander Exp $
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 * In other words, you are welcome to use, share and improve this program.
 * You are forbidden to forbid anyone else to use, share and improve
 * what you give them.   Happy coding!
 */

#ifndef SDF_BASE_TIMED_CLUSTER_H_INCLUDED
#define SDF_BASE_TIMED_CLUSTER_H_INCLUDED

#include "../untimed/graph.h"

/**
 * SDFactorCluster
 * Cluster of actors in SDF graph
 */
class SDFactorCluster : public SDFcomponent
{
public:

    // Constructor
    SDFactorCluster(SDFcomponent &c);
    
    // Desctructor
    ~SDFactorCluster();

    // Actors
    SDFactor *getActor(const CString &name);
    SDFactors &getActors() { return actors; };
    SDFactorsIter actorsBegin() { return actors.begin(); };
    SDFactorsIter actorsEnd() { return actors.end(); };
    SDFactorsCIter actorsBegin() const { return actors.begin(); };
    SDFactorsCIter actorsEnd() const { return actors.end(); };
    void addActor(SDFactor *a);
    void removeActor(const CString &name);
    
    // Graph
    SDFgraph *getGraph() const { return (SDFgraph*)getParent(); };

    // Print
    ostream &print(ostream &out);
    friend ostream &operator<<(ostream &out, SDFactorCluster &a)
        { return a.print(out); };
    
private:
    SDFactors actors;
};

typedef list<SDFactorCluster*>              SDFactorClusters;
typedef SDFactorClusters::iterator          SDFactorClustersIter;
typedef SDFactorClusters::const_iterator    SDFactorClustersCIter;

#endif
